/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_CREATIONFUNCTION_H
#define X3DTK_CREATIONFUNCTION_H

#include "X3DTypes.h"
#include "X3DFunction.h"
#include "X3D_SceneGraphTypes.h"

namespace X3DTK {

/*! \brief Type declared in <b><X3DTK/X3D/scenegraph.h></b> defining a pointer to a
 *  creation function.
 *  
 *  \ingroup X3DLoader
 */

typedef X3DAbstractNode *(* ptrToCreationFunction)();

/*! \brief Class declared in <b><X3DTK/X3D/scenegraph.h></b> defining a creation function
 *  implemented by a functor (Command design pattern).
 *  
 *  \ingroup X3DLoader
 */

class CreationFunction : public X3DFunction
{
public:
  /// Constructor.
  CreationFunction(const ptrToCreationFunction create);
  /// Destructor.
  ~CreationFunction();
  
  /// Creates the node from the pointed function
  inline X3DAbstractNode *create() const {return (*_ptr2function)();};
  
private:
  /// Pointed function.
  const ptrToCreationFunction _ptr2function;
};

}

#endif
