/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GL_RENDERER_H
#define X3DTK_GL_RENDERER_H

#include "GL_RendererStateVariables.h"
#include "X3DOnePassProcessor.h"

namespace X3DTK {
namespace GL {

/*! \brief Class declared in <b><X3DTK/GL/renderer.h></b> providing a processor that
 *  displays the scene in an OpenGL context.
 *
 *  The scene is traversed and rendered: Appearance properties are applied for 
 *  material or texture nodes, transforms are applied for transform nodes and 
 *  the draw method is called for X3DGeometryNode nodes.
 *
 *  \ingroup GL
 */
 
class Renderer : public X3DOnePassProcessor
{
public:
  /// Constructor.
  Renderer();
  /// Destructor.
  virtual ~Renderer();
  
  /// Renders a sub scene graph in an OpenGL context.
  virtual void render(X3DNode *N) const;
  /// Renders a sub scene graph in selection mode in an OpenGL context. Doesn't work!
  virtual void select(X3DNode *N, double x, double y) const;
};

}
}

#endif
