/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GL_UPDATER_H
#define X3DTK_GL_UPDATER_H

#include "GL_UpdaterStateVariables.h"
#include "X3DOnePassProcessor.h"
#include "GL_X3DNode.h"

namespace X3DTK {
namespace GL {

/*! \brief Class declared in <b><X3DTK/GL/updater.h></b> providing a processor that 
 *  updates the content of the GL scene graph from the X3D scene graph.
 *
 *  The nodes of the GL scene graph contain specific informations for the GL rendering 
 *  that need to be synchronized with those of the X3D scene graph. For example 
 *  the X3DTK::GL::IndexedFaceSet node contain vertex arrays which are updated from
 *  the X3DTK::X3D::IndexedFaceSet, X3DTK::X3D::Coordinate, X3DTK::X3D::Normal, 
 *  X3DTK::X3D::TextureCoordinate and X3DTK::X3D::Color.
 *
 *  \ingroup GL
 */
 
class Updater : public X3DOnePassProcessor
{
public:
  /// Constructor.
  Updater();
  /// Destructor.
  virtual ~Updater();
  
  /// Updates the sub scene graph.
  virtual void update(X3DNode *N);
};

}
}

#endif
