/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GRAPHTRAVERSAL_H
#define X3DTK_GRAPHTRAVERSAL_H

#include "Singleton.h"
#include "StateVariables.h"
#include "X3DTypes.h"

#include <list>
#include <typeinfo>
#include <iostream>

namespace X3DTK {

class Walker;
class Visitor; 
class X3DComponentVisitor;

/*! \brief Class declared in <b><X3DTK/kernel.h></b> defining a tree traversal
 *  algorithm.
 *
 *  \ingroup processor
 */
 
class GraphTraversal
{
public:
  /// Constructor.
  GraphTraversal();
  /// Destructor.  
  virtual ~GraphTraversal();
  
  /// Sets the walker. 
  void setWalker(Walker *walker);
  /// Sets a component visitor. If there is already a visitor recorded for this 
  /// component, then it is deleted and replaced by the new one.
  void setComponentVisitor(X3DComponentVisitor *component);
  /// If AutoDeleteComponents is set to true, then the components are deleted
  /// in the destructor.
  void setAutoDeleteComponents(bool value);
  /// Traverses the sub-graph of which the root is N. The recorded component visitors 
  /// are applied.
  void traverse(SFNode N) const;
    
protected:
  /// Visitor.
  Visitor *visitor;     
  /// Walker.
  Walker *walker;
};

}

#endif
