/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_EDGESDRAWER_H
#define X3DTK_MESH_EDGESDRAWER_H

#include "MESH_EdgesDrawerStateVariables.h"
#include "DFSGraphTraversal.h"
#include "MESH_EdgesDrawerCoreVisitor.h"
#include "MESH_SceneGraphTypes.h"
#include "MESH_Mesh.h"
#include "X3DOnePassProcessor.h"
#include "MESH_MeshData.h"

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/edgesdrawer.h></b> providing a processor that 
 *  draws the edges of the MESH scene graph.
 *
 *  The boundary edges are drawn in blue color, the rounding faces of the non-manifold 
 *  edges are drawn in green. The other edges are drawn in gray.
 *
 *  \ingroup MESH
 */
 
template<class MData, class VData, class EData, class FData, bool RW>
class TemplateEdgesDrawer : public X3DOnePassProcessor
{
public:
  /// Constructor.
  TemplateEdgesDrawer();
  /// Destructor.
  virtual ~TemplateEdgesDrawer();
 
  /// Draws the edges of the tree. Selection is used for GL selection mode.
  void draw(X3DNode *N, bool selection = false);
  
  // Get ith Vertex of the scene.
  inline TemplateVertex<MData, VData, EData, FData, RW> *getVertex(int i) const;
  // Gets stored transformation matrix associated with a given Mesh.
  inline const SFMatrix34f &getMatrix(int i) const;
};

/// Default TemplateEdgesDrawer.
typedef TemplateEdgesDrawer<MeshData, VertexData, EdgeData, FaceData, true> EdgesDrawer;

}
}

#include "MESH_EdgesDrawer.inl"

#endif

