/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3DFILEELEMENT_H
#define X3DTK_X3DFILEELEMENT_H

#include "X3DTypes.h"

namespace X3DTK {

/*! \brief Abstract class declared in <b><X3DTK/kernel.h></b> providing an interface
 *  for a file element.
 *  
 *  It is implemented in different versions for the xml parsers.
 *
 *  \ingroup base
 */

class X3DFileElement
{
public:
  /// Gets the name.
  virtual SFString getName() const = 0;
  /// Gets the attribute at index i.
  virtual SFString getAttribute(int i) const = 0;
  /// Gets the index of the attribute. Returns -1 if doesn't exist.
  virtual int getIndexAttribute(const SFString &attribute) const = 0;
  
protected:
  /// Constructor.
  X3DFileElement();
  /// Destructor.
  virtual ~X3DFileElement();
};

}

#endif
