/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_GLBUILDER_H
#define X3DTK_X3D_GLBUILDER_H

#include "X3D_GLBuilderStateVariables.h"
#include "X3DOnePassProcessor.h"
#include "X3D_Scene.h"
#include "GL_Scene.h"

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/glbuilder.h></b> providing a processor
 *  that builds a GL scene graph from an X3D scene graph.
 *
 *  Some informations are lost during the conversion. An X3DTK::GL::IndexedFaceSet
 *  is made up of triangular faces, whereas an X3DTK::X3D::IndexedFaceSet can have 
 *  polygonal faces. The resulting scene graph looks like the X3D one but 
 *  X3DTK::GL::IndexedFaceSet nodes have no child. For more details about the MESH scene graph, 
 *  see the \ref GLSceneGraphAPI "GL" scene graph page.
 *
 *  \ingroup X3D
 */

class GLBuilder : public X3DOnePassProcessor
{
public:
  /// Constructor.
  GLBuilder();
  /// Destructor.
  virtual ~GLBuilder();

  /// Converts the X3D sub scene graph of which root is N, to an GL scene graph.
  virtual GL::X3DNode *build(X3DNode *N);
  
  /// Builds the GL scene from S.
  GL::Scene *build(Scene *S);
};

}
}

#endif
