/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_INDEXEDLINESET_H
#define X3DTK_X3D_INDEXEDLINESET_H

#include "X3DTypes.h"
#include "X3D_X3DGeometryNode.h"

namespace X3DTK {
namespace X3D {

class X3DColorNode;
class X3DCoordinateNode;

/*! \brief Class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/rendering.html">
 *  Rendering component</a> defining an IndexedLineSet
 *  (<a href="../../doc/X3DSpecs/Part01/components/rendering.html#IndexedLineSet">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class IndexedLineSet : public X3DGeometryNode
{
public:
  /// Constructor.
  IndexedLineSet();
  /// Destructor.
  virtual ~IndexedLineSet();
  
  /// Sets the ColorIndex attribute.
  void setColorIndex(const MFInt32 &colorIndex);
  /// Sets the ColorPerVertex attribute.
  void setColorPerVertex(const SFBool &colorPerVertex);
  /// Sets the CoordIndex attribute.
  void setCoordIndex(const MFInt32 &coordIndex);
  /// Sets the LineWidth attribute.
  void setLineWidth(const SFFloat &lineWidth);
  
  /// Gets the Color attribute.
  inline X3DColorNode *getColor() const {return _color;};
  /// Gets the Coord attribute.
  inline X3DCoordinateNode *getCoord() const {return _coord;};
  /// Gets the ColorIndex attribute.
  inline const MFInt32 &getColorIndex() const {return _colorIndex;};
  /// Gets the ColorPerVertex attribute.
  inline const SFBool &getColorPerVertex() const {return _colorPerVertex;};
  /// Gets the CoordIndex attribute.
  inline const MFInt32 &getCoordIndex() const {return _coordIndex;};
  /// Gets the LineWidth attribute.
  inline const SFFloat &getLineWidth() const {return _lineWidth;};

private:  
  /// Color node.
  X3DColorNode *_color;
  /// Coord node.
  X3DCoordinateNode *_coord;
  /// Color index attribute.
  MFInt32 _colorIndex;
  /// Color per vertex attribute.
  SFBool _colorPerVertex;
  /// Coord index attribute.
  MFInt32 _coordIndex;
  /// Line width attribute.
  SFFloat _lineWidth;  
};

}
}

#endif
