/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_LINEPROPERTIES_H
#define X3DTK_X3D_LINEPROPERTIES_H

#include "X3DTypes.h"
#include "X3D_X3DAppearanceChildNode.h"

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/shape.html">shape 
 *  component</a> defining a LineProperties
 *  (<a href="../../doc/X3DSpecs/Part01/components/shape.html#LineProperties">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class LineProperties : public X3DAppearanceChildNode
{
public:
  /// Constructor.
  LineProperties();
  /// Destructor.
  virtual ~LineProperties();
  
  /// Sets the LineStyle attribute.
  void setLineStyle(const SFInt32 &lineStyle);
  /// Sets the Width attribute.
  void setWidth(const SFFloat &width);
  
  /// Gets the LineStyle attribute.
  inline const SFInt32 &getLineStyle() const {return _lineStyle;};
  /// Gets the Width attribute.
  inline const SFFloat &getWidth() const {return _width;};

private:    
  /// Line style.
  SFInt32 _lineStyle;
  /// Width.
  SFFloat _width;
};

}
}

#endif
