/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_WORLDCOORDTRANSLATORSTATEVARIABLES_H
#define X3DTK_X3D_WORLDCOORDTRANSLATORSTATEVARIABLES_H

#include "X3D_SceneGraphTypes.h"
#include "StateVariables.h"

#include <list>

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/worldcoordtranslator.h></b> providing StateVariables
 *  of the X3D::WorldCoordTranslator processor.
 *
 *  \ingroup X3D
 */

class WorldCoordTranslatorStateVariables : public StateVariables
{
public:
  /// Constructor.
  WorldCoordTranslatorStateVariables();
  /// Destructor.
  virtual ~WorldCoordTranslatorStateVariables();

  /// Initializes the traversal.
  void init();
  /// Finishes the traversal.
  void finish();

  /// Pushes the transformation matrix.
  void pushMatrix(const SFMatrix34f &transformation);  
  /// Pushes the Coordinate or Normal node.
  void pushProcessedNode(SFNode N);  

  /// Pushes the node.
  void pushNode(SFNode N);  
  /// Pops the node.
  void popNode();  
  /// Pops the transformation matrix.
  void popMatrix();
  
  /// Gets the parent of the node
  inline SFNode getParent() const {return _nodeStack.back();};
  
  /// Gets the current transformation matrix.
  SFMatrix34f getMatrix() const {return _matrixStack.front();};
  /// Returns TRUE if has already met the Coordinate.
  bool findProcessedNode(SFNode N);

private: 
  std::list<SFMatrix34f> _matrixStack;
  std::list<SFNode> _nodeList;
  std::list<SFNode> _nodeStack;
};

}
}

#endif
